@@ -1,3 +1,4 @@
+import os
 import warnings
 import re
 
@@ -5,7 +6,7 @@ import re
 from ._version import __version__
 
 from deepeval.event import track
-from deepeval.monitor import monitor, send_feedback
+from deepeval.monitor import monitor, a_monitor, send_feedback, a_send_feedback
 from deepeval.evaluate import evaluate, assert_test
 from deepeval.test_run import on_test_run_end, log_hyperparameters
 from deepeval.utils import login_with_confident_api_key
@@ -18,10 +19,12 @@ __all__ = [
     "log_hyperparameters",
     "track",
     "monitor",
+    "a_monitor",
+    "a_send_feedback",
+    "send_feedback",
     "evaluate",
     "assert_test",
     "on_test_run_end",
-    "send_feedback",
     "trace_langchain",
     "trace_llama_index",
     "confident_evaluate",
@@ -63,4 +66,9 @@ def check_for_update():
         pass
 
 
-check_for_update()
+def update_warning_opt_out():
+    return os.getenv("DEEPEVAL_UPDATE_WARNING_OPT_OUT") == "YES"
+
+
+if not update_warning_opt_out():
+    check_for_update()
