@@ -4,3 +4,7 @@ from ..mmlu.mmlu import MMLUTask
 from ..truthful_qa.truthful_qa import TruthfulQATask
 from ..human_eval.human_eval import HumanEvalTask
 from ..drop.drop import DROPTask
+from ..squad.squad import SQuADTask
+from ..math_qa.math_qa import MathQATask
+from ..logi_qa.logi_qa import LogiQATask
+from ..bbq.bbq import BBQTask
