@@ -16,6 +16,7 @@ from deepeval.dataset.utils import (
     convert_test_cases_to_goldens,
     convert_goldens_to_test_cases,
     convert_convo_goldens_to_convo_test_cases,
+    trimAndLoadJson,
 )
 from deepeval.dataset.api import (
     APIDataset,
@@ -23,21 +24,28 @@ from deepeval.dataset.api import (
     DatasetHttpResponse,
 )
 from deepeval.dataset.golden import Golden, ConversationalGolden
-from deepeval.test_case import LLMTestCase, ConversationalTestCase
+from deepeval.test_case import (
+    LLMTestCase,
+    ConversationalTestCase,
+    MLLMTestCase,
+    ToolCall,
+)
 from deepeval.utils import convert_keys_to_snake_case, is_confident
-from deepeval.synthesizer.types import *
 
 valid_file_types = ["csv", "json"]
 
 
 def validate_test_case_type(
-    test_case: Union[LLMTestCase, ConversationalTestCase], subject: str
+    test_case: Union[LLMTestCase, ConversationalTestCase, MLLMTestCase],
+    subject: str,
 ):
-    if not isinstance(test_case, LLMTestCase) and not isinstance(
-        test_case, ConversationalTestCase
+    if (
+        not isinstance(test_case, LLMTestCase)
+        and not isinstance(test_case, ConversationalTestCase)
+        and not isinstance(test_case, MLLMTestCase)
     ):
         raise TypeError(
-            f"Provided `{subject}` must be a list of LLMTestCase or ConversationalTestCase"
+            f"Provided `{subject}` must be a list of LLMTestCase, ConversationalTestCase, or MLLMTestCase"
         )
 
 
@@ -51,10 +59,13 @@ class EvaluationDataset:
     _conversational_test_cases: List[ConversationalTestCase] = field(
         default_factory=[], repr=None
     )
+    _confident_api_key: Optional[str] = None
 
     def __init__(
         self,
-        test_cases: List[Union[LLMTestCase, ConversationalTestCase]] = [],
+        test_cases: List[
+            Union[LLMTestCase, ConversationalTestCase, MLLMTestCase]
+        ] = [],
         goldens: List[Golden] = [],
         conversational_goldens: List[ConversationalGolden] = [],
     ):
@@ -67,6 +78,7 @@ class EvaluationDataset:
 
         llm_test_cases = []
         conversational_test_cases = []
+        mllm_test_cases = []
         for test_case in test_cases:
             if isinstance(test_case, LLMTestCase):
                 test_case._dataset_rank = len(llm_test_cases)
@@ -74,9 +86,13 @@ class EvaluationDataset:
             elif isinstance(test_case, ConversationalTestCase):
                 test_case._dataset_rank = len(conversational_test_cases)
                 conversational_test_cases.append(test_case)
+            elif isinstance(test_case, MLLMTestCase):
+                test_case._dataset_rank = len(mllm_test_cases)
+                mllm_test_cases.append(test_case)
 
         self._llm_test_cases = llm_test_cases
         self._conversational_test_cases = conversational_test_cases
+        self._mllm_test_cases = mllm_test_cases
 
     def __repr__(self):
         return (
@@ -87,21 +103,33 @@ class EvaluationDataset:
         )
 
     @property
-    def test_cases(self) -> List[Union[LLMTestCase, ConversationalTestCase]]:
-        return self._llm_test_cases + self._conversational_test_cases
+    def test_cases(
+        self,
+    ) -> List[Union[LLMTestCase, ConversationalTestCase, MLLMTestCase]]:
+        return (
+            self._llm_test_cases
+            + self._conversational_test_cases
+            + self._mllm_test_cases
+        )
 
     @test_cases.setter
     def test_cases(
-        self, test_cases: List[Union[LLMTestCase, ConversationalTestCase]]
+        self,
+        test_cases: List[
+            Union[LLMTestCase, ConversationalTestCase, MLLMTestCase]
+        ],
     ):
         if not isinstance(test_cases, list):
             raise TypeError("'test_cases' must be a list")
 
         llm_test_cases = []
         conversational_test_cases = []
+        mllm_test_cases = []
         for test_case in test_cases:
-            if not isinstance(test_case, LLMTestCase) and not isinstance(
-                test_case, ConversationalTestCase
+            if (
+                not isinstance(test_case, LLMTestCase)
+                and not isinstance(test_case, ConversationalTestCase)
+                and not isinstance(test_case, MLLMTestCase)
             ):
                 continue
 
@@ -113,12 +141,17 @@ class EvaluationDataset:
             elif isinstance(test_case, ConversationalTestCase):
                 test_case._dataset_rank = len(conversational_test_cases)
                 conversational_test_cases.append(test_case)
+            elif isinstance(test_case, MLLMTestCase):
+                test_case._dataset_rank = len(mllm_test_cases)
+                mllm_test_cases.append(test_case)
 
         self._llm_test_cases = llm_test_cases
         self._conversational_test_cases = conversational_test_cases
+        self._mllm_test_cases = mllm_test_cases
 
     def add_test_case(
-        self, test_case: Union[LLMTestCase, ConversationalTestCase]
+        self,
+        test_case: Union[LLMTestCase, ConversationalTestCase, MLLMTestCase],
     ):
         validate_test_case_type(test_case, subject="test cases")
 
@@ -130,6 +163,12 @@ class EvaluationDataset:
         elif isinstance(test_case, ConversationalTestCase):
             test_case._dataset_rank = len(self._conversational_test_cases)
             self._conversational_test_cases.append(test_case)
+        elif isinstance(test_case, MLLMTestCase):
+            test_case._dataset_rank = len(self._mllm_test_cases)
+            self._mllm_test_cases.append(test_case)
+
+    def __len__(self):
+        return len(self.test_cases)
 
     def __iter__(self):
         return iter(self.test_cases)
@@ -154,6 +193,10 @@ class EvaluationDataset:
         context_col_delimiter: str = ";",
         retrieval_context_col_name: Optional[str] = None,
         retrieval_context_col_delimiter: str = ";",
+        tools_called_col_name: Optional[str] = None,
+        tools_called_col_delimiter: str = ";",
+        expected_tools_col_name: Optional[str] = None,
+        expected_tools_col_delimiter: str = ";",
         additional_metadata_col_name: Optional[str] = None,
     ):
         """
@@ -198,6 +241,8 @@ class EvaluationDataset:
             )
 
         df = pd.read_csv(file_path)
+        # Convert np.nan (default for missing values in pandas) to None for compatibility with Python and Pydantic
+        df = df.astype(object).where(pd.notna(df), None)
 
         inputs = get_column_data(df, input_col_name)
         actual_outputs = get_column_data(df, actual_output_col_name)
@@ -218,6 +263,37 @@ class EvaluationDataset:
                 df, retrieval_context_col_name, default=""
             )
         ]
+        tools_called = []
+        for tools_called_json in get_column_data(
+            df, tools_called_col_name, default="[]"
+        ):
+            if tools_called_json:
+                try:
+                    parsed_tools = [
+                        ToolCall(**tool)
+                        for tool in trimAndLoadJson(tools_called_json)
+                    ]
+                    tools_called.append(parsed_tools)
+                except ValueError as e:
+                    raise ValueError(f"Error processing tools_called: {e}")
+            else:
+                tools_called.append([])
+
+        expected_tools = []
+        for expected_tools_json in get_column_data(
+            df, expected_tools_col_name, default="[]"
+        ):
+            if expected_tools_json:
+                try:
+                    parsed_tools = [
+                        ToolCall(**tool)
+                        for tool in trimAndLoadJson(expected_tools_json)
+                    ]
+                    expected_tools.append(parsed_tools)
+                except ValueError as e:
+                    raise ValueError(f"Error processing expected_tools: {e}")
+            else:
+                expected_tools.append([])
         additional_metadatas = [
             ast.literal_eval(metadata) if metadata else None
             for metadata in get_column_data(
@@ -231,6 +307,8 @@ class EvaluationDataset:
             expected_output,
             context,
             retrieval_context,
+            tools_called,
+            expected_tools,
             additional_metadata,
         ) in zip(
             inputs,
@@ -238,6 +316,8 @@ class EvaluationDataset:
             expected_outputs,
             contexts,
             retrieval_contexts,
+            tools_called,
+            expected_tools,
             additional_metadatas,
         ):
             self.add_test_case(
@@ -247,6 +327,8 @@ class EvaluationDataset:
                     expected_output=expected_output,
                     context=context,
                     retrieval_context=retrieval_context,
+                    tools_called=tools_called,
+                    expected_tools=expected_tools,
                     additional_metadata=additional_metadata,
                 )
             )
@@ -259,6 +341,9 @@ class EvaluationDataset:
         expected_output_key_name: Optional[str] = None,
         context_key_name: Optional[str] = None,
         retrieval_context_key_name: Optional[str] = None,
+        tools_called_key_name: Optional[str] = None,
+        expected_tools_key_name: Optional[str] = None,
+        encoding_type: str = "utf-8",
     ):
         """
         Load test cases from a JSON file.
@@ -284,7 +369,7 @@ class EvaluationDataset:
             The JSON file should be structured as a list of objects, with each object containing the required keys. The method assumes the file format and keys are correctly defined and present.
         """
         try:
-            with open(file_path, "r") as file:
+            with open(file_path, "r", encoding=encoding_type) as file:
                 json_list = json.load(file)
         except FileNotFoundError:
             raise FileNotFoundError(f"The file {file_path} was not found.")
@@ -306,6 +391,10 @@ class EvaluationDataset:
             expected_output = json_obj.get(expected_output_key_name)
             context = json_obj.get(context_key_name)
             retrieval_context = json_obj.get(retrieval_context_key_name)
+            tools_called_data = json_obj.get(tools_called_key_name, [])
+            tools_called = [ToolCall(**tool) for tool in tools_called_data]
+            expected_tools_data = json_obj.get(expected_tools_key_name, [])
+            expected_tools = [ToolCall(**tool) for tool in expected_tools_data]
 
             self.add_test_case(
                 LLMTestCase(
@@ -314,17 +403,200 @@ class EvaluationDataset:
                     expected_output=expected_output,
                     context=context,
                     retrieval_context=retrieval_context,
+                    tools_called=tools_called,
+                    expected_tools=expected_tools,
                 )
             )
 
-    def push(self, alias: str, overwrite: Optional[bool] = None):
-        if len(self.test_cases) == 0 and len(self.goldens) == 0:
-            raise ValueError(
-                "Unable to push empty dataset to Confident AI, there must be at least one test case or golden in dataset"
+    def add_goldens_from_csv_file(
+        self,
+        file_path: str,
+        input_col_name: str,
+        actual_output_col_name: str,
+        expected_output_col_name: Optional[str] = None,
+        context_col_name: Optional[str] = None,
+        context_col_delimiter: str = ";",
+        retrieval_context_col_name: Optional[str] = None,
+        retrieval_context_col_delimiter: str = ";",
+        tools_called_col_name: Optional[str] = None,
+        tools_called_col_delimiter: str = ";",
+        expected_tools_col_name: Optional[str] = None,
+        expected_tools_col_delimiter: str = ";",
+        source_file_col_name: Optional[str] = None,
+        additional_metadata_col_name: Optional[str] = None,
+    ):
+        try:
+            import pandas as pd
+        except ModuleNotFoundError:
+            raise ModuleNotFoundError(
+                "Please install pandas to use this method. 'pip install pandas'"
             )
+
+        def get_column_data(df: pd.DataFrame, col_name: str, default=None):
+            return (
+                df[col_name].values
+                if col_name in df.columns
+                else [default] * len(df)
+            )
+
+        df = (
+            pd.read_csv(file_path)
+            .astype(object)
+            .where(pd.notna(pd.read_csv(file_path)), None)
+        )
+
+        inputs = get_column_data(df, input_col_name)
+        actual_outputs = get_column_data(df, actual_output_col_name)
+        expected_outputs = get_column_data(
+            df, expected_output_col_name, default=None
+        )
+        contexts = [
+            context.split(context_col_delimiter) if context else []
+            for context in get_column_data(df, context_col_name, default="")
+        ]
+        retrieval_contexts = [
+            (
+                retrieval_context.split(retrieval_context_col_delimiter)
+                if retrieval_context
+                else []
+            )
+            for retrieval_context in get_column_data(
+                df, retrieval_context_col_name, default=""
+            )
+        ]
+        tools_called = [
+            (
+                tool_called.split(tools_called_col_delimiter)
+                if tool_called
+                else []
+            )
+            for tool_called in get_column_data(
+                df, tools_called_col_name, default=""
+            )
+        ]
+        expected_tools = [
+            (
+                expected_tool.split(expected_tools_col_delimiter)
+                if expected_tool
+                else []
+            )
+            for expected_tool in get_column_data(
+                df, expected_tools_col_name, default=""
+            )
+        ]
+        source_files = get_column_data(df, source_file_col_name)
+        additional_metadatas = [
+            ast.literal_eval(metadata) if metadata else None
+            for metadata in get_column_data(
+                df, additional_metadata_col_name, default=""
+            )
+        ]
+
+        for (
+            input,
+            actual_output,
+            expected_output,
+            context,
+            retrieval_context,
+            tools_called,
+            expected_tools,
+            source_file,
+            additional_metadata,
+        ) in zip(
+            inputs,
+            actual_outputs,
+            expected_outputs,
+            contexts,
+            retrieval_contexts,
+            tools_called,
+            expected_tools,
+            source_files,
+            additional_metadatas,
+        ):
+            self.goldens.append(
+                Golden(
+                    input=input,
+                    actual_output=actual_output,
+                    expected_output=expected_output,
+                    context=context,
+                    retrieval_context=retrieval_context,
+                    tools_called=tools_called,
+                    expected_tools=expected_tools,
+                    additional_metadata=additional_metadata,
+                    source_file=source_file,
+                )
+            )
+
+    def add_goldens_from_json_file(
+        self,
+        file_path: str,
+        input_key_name: str,
+        actual_output_key_name: str,
+        expected_output_key_name: Optional[str] = None,
+        context_key_name: Optional[str] = None,
+        retrieval_context_key_name: Optional[str] = None,
+        tools_called_key_name: Optional[str] = None,
+        expected_tools_key_name: Optional[str] = None,
+        source_file_key_name: Optional[str] = None,
+        encoding_type: str = "utf-8",
+    ):
+        try:
+            with open(file_path, "r", encoding=encoding_type) as file:
+                json_list = json.load(file)
+        except FileNotFoundError:
+            raise FileNotFoundError(f"The file {file_path} was not found.")
+        except json.JSONDecodeError:
+            raise ValueError(f"The file {file_path} is not a valid JSON file.")
+
+        # Process each JSON object
+        for json_obj in json_list:
+            if input_key_name not in json_obj:
+                raise ValueError(
+                    "Required fields are missing in one or more JSON objects"
+                )
+
+            input = json_obj[input_key_name]
+            actual_output = json_obj.get(actual_output_key_name)
+            expected_output = json_obj.get(expected_output_key_name)
+            context = json_obj.get(context_key_name)
+            retrieval_context = json_obj.get(retrieval_context_key_name)
+            tools_called = json_obj.get(tools_called_key_name)
+            expected_tools = json_obj.get(expected_tools_key_name)
+            source_file = json_obj.get(source_file_key_name)
+
+            self.goldens.append(
+                Golden(
+                    input=input,
+                    actual_output=actual_output,
+                    expected_output=expected_output,
+                    context=context,
+                    retrieval_context=retrieval_context,
+                    tools_called=tools_called,
+                    expected_tools=expected_tools,
+                    source_file=source_file,
+                )
+            )
+
+    def push(
+        self,
+        alias: str,
+        overwrite: Optional[bool] = None,
+        auto_convert_test_cases_to_goldens: bool = False,
+    ):
+        if auto_convert_test_cases_to_goldens is False:
+            if len(self.goldens) == 0:
+                raise ValueError(
+                    "Unable to push empty dataset to Confident AI, there must be at least one golden in dataset. To include test cases, set 'auto_convert_test_cases_to_goldens' to True."
+                )
+        else:
+            if len(self.test_cases) == 0 and len(self.goldens) == 0:
+                raise ValueError(
+                    "Unable to push empty dataset to Confident AI, there must be at least one test case or golden in dataset"
+                )
         if is_confident():
             goldens = self.goldens
-            goldens.extend(convert_test_cases_to_goldens(self.test_cases))
+            if auto_convert_test_cases_to_goldens:
+                goldens.extend(convert_test_cases_to_goldens(self.test_cases))
             api_dataset = APIDataset(
                 alias=alias,
                 overwrite=overwrite,
@@ -360,8 +632,8 @@ class EvaluationDataset:
             )
 
     def pull(self, alias: str, auto_convert_goldens_to_test_cases: bool = True):
-        if is_confident():
-            api = Api()
+        if is_confident() or self._confident_api_key is not None:
+            api = Api(api_key=self._confident_api_key)
             with Progress(
                 SpinnerColumn(style="rgb(106,0,255)"),
                 TextColumn("[progress.description]{task.description}"),
@@ -383,7 +655,7 @@ class EvaluationDataset:
                     result["conversationalGoldens"]
                 ):
                     if "goldens" in cg:
-                        cg["messages"] = cg.pop("goldens")
+                        cg["turns"] = cg.pop("goldens")
                     conversational_goldens.append(ConversationalGolden(**cg))
 
                 response = DatasetHttpResponse(
@@ -394,6 +666,9 @@ class EvaluationDataset:
 
                 self._alias = alias
                 self._id = response.datasetId
+                self.goldens = []
+                self.conversational_goldens = []
+                self.test_cases = []
 
                 if auto_convert_goldens_to_test_cases:
                     llm_test_cases = convert_goldens_to_test_cases(
@@ -427,122 +702,76 @@ class EvaluationDataset:
                 "Run `deepeval login` to pull dataset from Confident AI"
             )
 
-    def generate_red_teaming_goldens(
-        self,
-        synthesizer=None,
-        contexts: Optional[List[List[str]]] = None,
-        include_expected_output: bool = False,
-        max_goldens: int = 2,
-        num_evolutions: int = 1,
-        attacks: List[RTAdversarialAttack] = [
-            RTAdversarialAttack.PROMPT_INJECTION,
-            RTAdversarialAttack.PROMPT_PROBING,
-            RTAdversarialAttack.GRAY_BOX_ATTACK,
-            RTAdversarialAttack.JAILBREAKING,
-        ],
-        vulnerabilities: List[RTVulnerability] = [
-            RTVulnerability.BIAS,
-            RTVulnerability.DATA_LEAKAGE,
-            RTVulnerability.HALLUCINATION,
-            RTVulnerability.OFFENSIVE,
-            RTVulnerability.UNFORMATTED,
-        ],
-        use_case: UseCase = UseCase.QA,
-        _show_indicator: bool = True,
-    ):
-        from deepeval.synthesizer import Synthesizer
-
-        if synthesizer is None:
-            synthesizer = Synthesizer()
-        else:
-            assert isinstance(synthesizer, Synthesizer)
-
-        self.goldens.extend(
-            synthesizer.generate_red_teaming_goldens(
-                contexts=contexts,
-                include_expected_output=include_expected_output,
-                max_goldens=max_goldens,
-                num_evolutions=num_evolutions,
-                attacks=attacks,
-                _show_indicator=_show_indicator,
-                vulnerabilities=vulnerabilities,
-                use_case=use_case,
-            )
-        )
-
-    def generate_goldens(
-        self,
-        contexts: List[List[str]],
-        include_expected_output: bool = False,
-        max_goldens_per_context: int = 2,
-        num_evolutions: int = 1,
-        source_files: Optional[List[str]] = None,
-        evolutions: List[Evolution] = [
-            Evolution.REASONING,
-            Evolution.MULTICONTEXT,
-            Evolution.CONCRETIZING,
-            Evolution.CONSTRAINED,
-            Evolution.COMPARATIVE,
-            Evolution.HYPOTHETICAL,
-            Evolution.IN_BREADTH,
-        ],
-        use_case: UseCase = UseCase.QA,
-        synthesizer=None,
-    ):
-        from deepeval.synthesizer import Synthesizer
-
-        if synthesizer is None:
-            synthesizer = Synthesizer()
-        else:
-            assert isinstance(synthesizer, Synthesizer)
-
-        self.goldens.extend(
-            synthesizer.generate_goldens(
-                contexts=contexts,
-                include_expected_output=include_expected_output,
-                max_goldens_per_context=max_goldens_per_context,
-                num_evolutions=num_evolutions,
-                source_files=source_files,
-                evolutions=evolutions,
-                use_case=use_case,
-            )
-        )
-
     def generate_goldens_from_docs(
         self,
         document_paths: List[str],
-        include_expected_output: bool = False,
-        max_goldens_per_document: int = 5,
-        chunk_size: int = 1024,
-        chunk_overlap: int = 0,
-        num_evolutions: int = 1,
-        evolutions: List[Evolution] = [
-            Evolution.REASONING,
-            Evolution.MULTICONTEXT,
-            Evolution.CONCRETIZING,
-            Evolution.CONSTRAINED,
-            Evolution.COMPARATIVE,
-            Evolution.HYPOTHETICAL,
-            Evolution.IN_BREADTH,
-        ],
+        include_expected_output: bool = True,
+        max_goldens_per_context: int = 2,
+        context_construction_config=None,
         synthesizer=None,
     ):
         from deepeval.synthesizer import Synthesizer
+        from deepeval.synthesizer.config import ContextConstructionConfig
 
         if synthesizer is None:
             synthesizer = Synthesizer()
         else:
             assert isinstance(synthesizer, Synthesizer)
 
+        if context_construction_config is not None:
+            assert isinstance(
+                context_construction_config, ContextConstructionConfig
+            )
+
         self.goldens.extend(
             synthesizer.generate_goldens_from_docs(
                 document_paths=document_paths,
                 include_expected_output=include_expected_output,
-                max_goldens_per_document=max_goldens_per_document,
-                chunk_size=chunk_size,
-                chunk_overlap=chunk_overlap,
-                num_evolutions=num_evolutions,
-                evolutions=evolutions,
+                max_goldens_per_context=max_goldens_per_context,
+                context_construction_config=context_construction_config,
+                _send_data=False,
+            )
+        )
+
+    def generate_goldens_from_contexts(
+        self,
+        contexts: List[List[str]],
+        include_expected_output: bool = True,
+        max_goldens_per_context: int = 2,
+        synthesizer=None,
+    ):
+        from deepeval.synthesizer import Synthesizer
+
+        if synthesizer is None:
+            synthesizer = Synthesizer()
+        else:
+            assert isinstance(synthesizer, Synthesizer)
+
+        self.goldens.extend(
+            synthesizer.generate_goldens_from_contexts(
+                contexts=contexts,
+                include_expected_output=include_expected_output,
+                max_goldens_per_context=max_goldens_per_context,
+                _send_data=False,
+            )
+        )
+
+    def generate_goldens_from_scratch(
+        self,
+        num_goldens: int,
+        synthesizer=None,
+    ):
+        from deepeval.synthesizer import Synthesizer
+
+        if synthesizer is None:
+            synthesizer = Synthesizer()
+        else:
+            assert isinstance(synthesizer, Synthesizer)
+
+        self.goldens.extend(
+            synthesizer.generate_goldens_from_scratch(
+                num_goldens=num_goldens,
+                _send_data=False,
             )
         )
 
@@ -568,7 +797,7 @@ class EvaluationDataset:
         full_file_path = os.path.join(directory, new_filename)
 
         if file_type == "json":
-            with open(full_file_path, "w") as file:
+            with open(full_file_path, "w", encoding="utf-8") as file:
                 json_data = [
                     {
                         "input": golden.input,
@@ -579,10 +808,12 @@ class EvaluationDataset:
                     }
                     for golden in self.goldens
                 ]
-                json.dump(json_data, file, indent=4)
+                json.dump(json_data, file, indent=4, ensure_ascii=False)
 
         elif file_type == "csv":
-            with open(full_file_path, "w", newline="") as file:
+            with open(
+                full_file_path, "w", newline="", encoding="utf-8"
+            ) as file:
                 writer = csv.writer(file)
                 writer.writerow(
                     [
