@@ -1,5 +1,6 @@
 from pydantic import BaseModel, Field
 from typing import Optional, Dict, List
+from deepeval.test_case import ToolCall
 
 
 class Golden(BaseModel):
@@ -18,6 +19,12 @@ class Golden(BaseModel):
         default=None, serialization_alias="additionalMetadata"
     )
     comments: Optional[str] = Field(default=None)
+    tools_called: Optional[List[ToolCall]] = Field(
+        default=None, serialization_alias="toolsCalled"
+    )
+    expected_tools: Optional[List[ToolCall]] = Field(
+        default=None, serialization_alias="expectedTools"
+    )
     source_file: Optional[str] = Field(
         default=None, serialization_alias="sourceFile"
     )
@@ -28,7 +35,7 @@ class ConversationalGolden(BaseModel):
         default=None, serialization_alias="additionalMetadata"
     )
     comments: Optional[str] = Field(default=None)
-    messages: List[Golden] = Field(
+    turns: List[Golden] = Field(
         default_factory=lambda: [],
         serialization_alias="goldens",
     )
