@@ -12,7 +12,6 @@ from typing import (
 )
 from time import perf_counter
 import json
-import traceback
 
 from llama_index.core.bridge.pydantic import BaseModel
 from llama_index.core.callbacks.base_handler import BaseCallbackHandler
@@ -26,10 +25,10 @@ from llama_index.core.callbacks.base_handler import BaseCallbackHandler
 from llama_index.core.callbacks.schema import BASE_TRACE_EVENT
 from llama_index.core.llms import ChatMessage, ChatResponse
 from llama_index.core.tools import ToolMetadata
+from openai.types.chat import ChatCompletion
 
 from deepeval.tracing import (
     trace_manager,
-    get_trace_stack,
     BaseTrace,
     LlmTrace,
     GenericTrace,
@@ -59,7 +58,7 @@ from deepeval.tracing import (
     TraceData,
 )
 from deepeval.utils import dataclass_to_dict, class_to_dict
-from deepeval.event import track
+from deepeval.monitor import monitor
 
 events_to_ignore = [
     # CBEventType.CHUNKING,
@@ -156,7 +155,7 @@ class LlamaIndexCallbackHandler(BaseCallbackHandler):
             trace_manager.clear_trace_stack()
 
             if trace_manager.get_outter_provider() == TraceProvider.LLAMA_INDEX:
-                track(
+                monitor(
                     event_name=current_trace_stack[0].name,
                     model=self.track_params.get("model") or "NA",
                     input=self.track_params.get("input") or "NA",
@@ -201,7 +200,7 @@ class LlamaIndexCallbackHandler(BaseCallbackHandler):
         elif event_type == CBEventType.AGENT_STEP:
             trace_instance = AgentTrace(
                 **trace_kwargs,
-                embeddingAttributes=AgentAttributes(
+                agentAttributes=AgentAttributes(
                     input="", output="", name="", description=""
                 ),
             )
@@ -364,7 +363,10 @@ class LlamaIndexCallbackHandler(BaseCallbackHandler):
             ]
             for node in nodes:
                 total_chunk_length += len(node.content)
-                top_score = node.score if node.score > top_score else top_score
+                if node.score:
+                    top_score = (
+                        node.score if node.score > top_score else top_score
+                    )
             attributes.nodes = nodes
             attributes.top_k = len(nodes)
             attributes.average_chunk_size = total_chunk_length // len(nodes)
@@ -510,13 +512,36 @@ class LlamaIndexCallbackHandler(BaseCallbackHandler):
             attributes.update(self._get_response_output(response))
             if hasattr(response, "raw"):
                 raw = response.raw
-                assert isinstance(
-                    raw, Mapping
-                ), f"raw must be Mapping, found {type(raw)}"
-                attributes.update(self._get_output_messages(raw))
-                if "usage" in raw:
-                    attributes.update(self._get_token_counts(raw["usage"]))
-
+                if isinstance(raw, Mapping):
+                    attributes.update(self._get_output_messages(raw))
+                    if "usage" in raw:
+                        attributes.update(self._get_token_counts(raw["usage"]))
+                elif isinstance(
+                    raw, ChatCompletion
+                ):  # Handle the ChatCompletion case
+                    if hasattr(raw, "choices") and hasattr(raw, "usage"):
+                        # Extract data from ChatCompletion object
+                        attributes.update(
+                            {
+                                "id": raw.id,
+                                "created": raw.created,
+                                "model": raw.model,
+                                "usage": {
+                                    "completion_tokens": raw.usage.completion_tokens,
+                                    "prompt_tokens": raw.usage.prompt_tokens,
+                                    "total_tokens": raw.usage.total_tokens,
+                                },
+                                "response": (
+                                    raw.choices[0].message.content
+                                    if raw.choices
+                                    else None
+                                ),
+                            }
+                        )
+                    else:
+                        print(f"Unexpected object structure in raw: {raw}")
+                else:
+                    print(f"Unexpected raw type: {type(raw)}")
             if (
                 additional_kwargs := getattr(
                     response, "additional_kwargs", None
