@@ -4,6 +4,7 @@ import asyncio
 from deepeval.test_case import (
     LLMTestCase,
     LLMTestCaseParams,
+    ConversationalTestCase,
 )
 from deepeval.metrics import BaseMetric
 from deepeval.models import DeepEvalBaseLLM
@@ -37,6 +38,7 @@ class SummarizationMetric(BaseMetric):
         async_mode=True,
         strict_mode: bool = False,
         verbose_mode: bool = False,
+        truths_extraction_limit: Optional[int] = None,
     ):
         self.threshold = 1 if strict_mode else threshold
         self.model, self.using_native_model = initialize_model(model)
@@ -53,9 +55,17 @@ class SummarizationMetric(BaseMetric):
         self.strict_mode = strict_mode
         self.verbose_mode = verbose_mode
 
+        self.truths_extraction_limit = truths_extraction_limit
+        if self.truths_extraction_limit is not None:
+            self.truths_extraction_limit = max(self.truths_extraction_limit, 0)
+
     def measure(
-        self, test_case: LLMTestCase, _show_indicator: bool = True
+        self,
+        test_case: Union[LLMTestCase, ConversationalTestCase],
+        _show_indicator: bool = True,
     ) -> float:
+        if isinstance(test_case, ConversationalTestCase):
+            test_case = test_case.turns[0]
         check_llm_test_case_params(test_case, required_params, self)
 
         self.evaluation_cost = 0 if self.using_native_model else None
@@ -66,7 +76,7 @@ class SummarizationMetric(BaseMetric):
                     self.a_measure(test_case, _show_indicator=False)
                 )
             else:
-                self.truths: List[str] = self._generate_claims(test_case.input)
+                self.truths: List[str] = self._generate_truths(test_case.input)
                 self.claims: List[str] = self._generate_claims(
                     test_case.actual_output
                 )
@@ -88,7 +98,7 @@ class SummarizationMetric(BaseMetric):
                 self.verbose_logs = construct_verbose_logs(
                     self,
                     steps=[
-                        f"Truths:\n{prettify_list(self.truths)}",
+                        f"Truths (limit={self.truths_extraction_limit}):\n{prettify_list(self.truths)}",
                         f"Claims:\n{prettify_list(self.claims)}",
                         f"Assessment Questions:\n{prettify_list(self.assessment_questions)}",
                         f"Coverage Verdicts:\n{prettify_list(self.coverage_verdicts)}",
@@ -101,9 +111,11 @@ class SummarizationMetric(BaseMetric):
 
     async def a_measure(
         self,
-        test_case: LLMTestCase,
+        test_case: Union[LLMTestCase, ConversationalTestCase],
         _show_indicator: bool = True,
     ) -> float:
+        if isinstance(test_case, ConversationalTestCase):
+            test_case = test_case.turns[0]
         check_llm_test_case_params(test_case, required_params, self)
 
         self.evaluation_cost = 0 if self.using_native_model else None
@@ -113,7 +125,7 @@ class SummarizationMetric(BaseMetric):
             _show_indicator=_show_indicator,
         ):
             self.truths, self.claims = await asyncio.gather(
-                self._a_generate_claims(test_case.input),
+                self._a_generate_truths(test_case.input),
                 self._a_generate_claims(test_case.actual_output),
             )
             (
@@ -135,7 +147,7 @@ class SummarizationMetric(BaseMetric):
             self.verbose_logs = construct_verbose_logs(
                 self,
                 steps=[
-                    f"Truths:\n{prettify_list(self.truths)}",
+                    f"Truths (limit={self.truths_extraction_limit}):\n{prettify_list(self.truths)}",
                     f"Claims:\n{prettify_list(self.claims)}",
                     f"Assessment Questions:\n{prettify_list(self.assessment_questions)}",
                     f"Coverage Verdicts:\n{prettify_list(self.coverage_verdicts)}",
@@ -183,10 +195,9 @@ class SummarizationMetric(BaseMetric):
 """
 
         if self.using_native_model:
-            res, cost = await self.model.a_generate(prompt)
+            res, cost = await self.model.a_generate(prompt, schema=Reason)
             self.evaluation_cost += cost
-            data = trimAndLoadJson(res, self)
-            return data["reason"]
+            return res.reason
         else:
             try:
                 res: Reason = await self.model.a_generate(prompt, schema=Reason)
@@ -233,10 +244,9 @@ class SummarizationMetric(BaseMetric):
 """
 
         if self.using_native_model:
-            res, cost = self.model.generate(prompt)
+            res, cost = self.model.generate(prompt, schema=Reason)
             self.evaluation_cost += cost
-            data = trimAndLoadJson(res, self)
-            return data["reason"]
+            return res.reason
         else:
             try:
                 res: Reason = self.model.generate(prompt, schema=Reason)
@@ -283,10 +293,9 @@ class SummarizationMetric(BaseMetric):
             questions=self.assessment_questions, text=text
         )
         if self.using_native_model:
-            res, cost = await self.model.a_generate(prompt)
+            res, cost = await self.model.a_generate(prompt, schema=Answers)
             self.evaluation_cost += cost
-            data = trimAndLoadJson(res, self)
-            return data["answers"]
+            return res.answers
         else:
             try:
                 res: Answers = await self.model.a_generate(
@@ -303,10 +312,9 @@ class SummarizationMetric(BaseMetric):
             questions=self.assessment_questions, text=text
         )
         if self.using_native_model:
-            res, cost = self.model.generate(prompt)
+            res, cost = self.model.generate(prompt, schema=Answers)
             self.evaluation_cost += cost
-            data = trimAndLoadJson(res, self)
-            return data["answers"]
+            return res.answers
         else:
             try:
                 res: Answers = self.model.generate(prompt, schema=Answers)
@@ -319,10 +327,9 @@ class SummarizationMetric(BaseMetric):
     async def _a_generate_assessment_questions(self, text: str):
         prompt = SummarizationTemplate.generate_questions(text=text, n=self.n)
         if self.using_native_model:
-            res, cost = await self.model.a_generate(prompt)
+            res, cost = await self.model.a_generate(prompt, schema=Questions)
             self.evaluation_cost += cost
-            data = trimAndLoadJson(res, self)
-            return data["questions"]
+            return res.questions
         else:
             try:
                 res: Questions = await self.model.a_generate(
@@ -337,10 +344,9 @@ class SummarizationMetric(BaseMetric):
     def _generate_assessment_questions(self, text: str):
         prompt = SummarizationTemplate.generate_questions(text=text, n=self.n)
         if self.using_native_model:
-            res, cost = self.model.generate(prompt)
+            res, cost = self.model.generate(prompt, schema=Questions)
             self.evaluation_cost += cost
-            data = trimAndLoadJson(res, self)
-            return data["questions"]
+            return res.questions
         else:
             try:
                 res: Questions = self.model.generate(prompt, schema=Questions)
@@ -417,13 +423,9 @@ class SummarizationMetric(BaseMetric):
             summary_claims=self.claims, orignal_text="\n\n".join(self.truths)
         )
         if self.using_native_model:
-            res, cost = await self.model.a_generate(prompt)
+            res, cost = await self.model.a_generate(prompt, schema=Verdicts)
             self.evaluation_cost += cost
-            data = trimAndLoadJson(res, self)
-            verdicts = [
-                SummarizationAlignmentVerdict(**item)
-                for item in data["verdicts"]
-            ]
+            verdicts = [item for item in res.verdicts]
             return verdicts
         else:
             try:
@@ -452,13 +454,9 @@ class SummarizationMetric(BaseMetric):
             summary_claims=self.claims, orignal_text="\n\n".join(self.truths)
         )
         if self.using_native_model:
-            res, cost = self.model.generate(prompt)
+            res, cost = self.model.generate(prompt, schema=Verdicts)
             self.evaluation_cost += cost
-            data = trimAndLoadJson(res, self)
-            verdicts = [
-                SummarizationAlignmentVerdict(**item)
-                for item in data["verdicts"]
-            ]
+            verdicts = [item for item in res.verdicts]
             return verdicts
         else:
             try:
@@ -474,14 +472,31 @@ class SummarizationMetric(BaseMetric):
                 ]
                 return verdicts
 
+    async def _a_generate_truths(self, text: str) -> List[str]:
+        # Borrow faithfulness template
+        prompt = FaithfulnessTemplate.generate_truths(
+            text=text, extraction_limit=self.truths_extraction_limit
+        )
+        if self.using_native_model:
+            res, cost = await self.model.a_generate(prompt, schema=Truths)
+            self.evaluation_cost += cost
+            return res.truths
+        else:
+            try:
+                res: Truths = await self.model.a_generate(prompt, schema=Truths)
+                return res.truths
+            except TypeError:
+                res = await self.model.a_generate(prompt)
+                data = trimAndLoadJson(res, self)
+                return data["truths"]
+
     async def _a_generate_claims(self, text: str) -> List[str]:
         # Borrow faithfulness template
         prompt = FaithfulnessTemplate.generate_claims(text=text)
         if self.using_native_model:
-            res, cost = await self.model.a_generate(prompt)
+            res, cost = await self.model.a_generate(prompt, schema=Claims)
             self.evaluation_cost += cost
-            data = trimAndLoadJson(res, self)
-            return data["claims"]
+            return res.claims
         else:
             try:
                 res: Claims = await self.model.a_generate(prompt, schema=Claims)
@@ -491,14 +506,31 @@ class SummarizationMetric(BaseMetric):
                 data = trimAndLoadJson(res, self)
                 return data["claims"]
 
+    def _generate_truths(self, text: str) -> List[str]:
+        # Borrow faithfulness template
+        prompt = FaithfulnessTemplate.generate_truths(
+            text=text, extraction_limit=self.truths_extraction_limit
+        )
+        if self.using_native_model:
+            res, cost = self.model.generate(prompt, schema=Truths)
+            self.evaluation_cost += cost
+            return res.truths
+        else:
+            try:
+                res: Truths = self.model.generate(prompt, schema=Truths)
+                return res.truths
+            except TypeError:
+                res = self.model.generate(prompt)
+                data = trimAndLoadJson(res, self)
+                return data["truths"]
+
     def _generate_claims(self, text: str) -> List[str]:
         # Borrow faithfulness template
         prompt = FaithfulnessTemplate.generate_claims(text=text)
         if self.using_native_model:
-            res, cost = self.model.generate(prompt)
+            res, cost = self.model.generate(prompt, schema=Claims)
             self.evaluation_cost += cost
-            data = trimAndLoadJson(res, self)
-            return data["claims"]
+            return res.claims
         else:
             try:
                 res: Claims = self.model.generate(prompt, schema=Claims)
