@@ -1,9 +1,11 @@
 from deepeval.models.base_model import (
     DeepEvalBaseModel,
     DeepEvalBaseLLM,
+    DeepEvalBaseMLLM,
     DeepEvalBaseEmbeddingModel,
 )
-from deepeval.models.gpt_model import GPTModel
+from deepeval.models.gpt_model import GPTModel, MultimodalGPTModel
+from deepeval.models.gpt_model_schematic import SchematicGPTModel
 from deepeval.models.openai_embedding_model import OpenAIEmbeddingModel
 
 # TODO: uncomment out once fixed
