@@ -18,7 +18,7 @@ from deepeval.test_run.api import (
     ConversationalApiTestCase,
     TestRunHttpResponse,
 )
-from deepeval.test_case import LLMTestCase, ConversationalTestCase
+from deepeval.test_case import LLMTestCase, ConversationalTestCase, MLLMTestCase
 from deepeval.utils import (
     delete_file_if_exists,
     get_is_running_deepeval,
@@ -91,6 +91,7 @@ class TestRun(BaseModel):
     metrics_scores: List[MetricScores] = Field(
         default_factory=lambda: [], alias="metricsScores"
     )
+    identifier: Optional[str] = None
     hyperparameters: Optional[Dict[Any, Any]] = Field(None)
     test_passed: Optional[int] = Field(None, alias="testPassed")
     test_failed: Optional[int] = Field(None, alias="testFailed")
@@ -111,18 +112,22 @@ class TestRun(BaseModel):
                         api_test_case.conversational_instance_id
                         == conversational_test_case.instance_id
                     ):
-                        conversational_test_case.messages[
-                            api_test_case.order
-                        ] = api_test_case
+                        conversational_test_case.turns[api_test_case.order] = (
+                            api_test_case
+                        )
+
+                        if api_test_case.success is False:
+                            conversational_test_case.success = False
 
                         if conversational_test_case.evaluation_cost is None:
                             conversational_test_case.evaluation_cost = (
                                 api_test_case.evaluation_cost
                             )
                         else:
-                            conversational_test_case.evaluation_cost += (
-                                api_test_case.evaluation_cost
-                            )
+                            if api_test_case.evaluation_cost is not None:
+                                conversational_test_case.evaluation_cost += (
+                                    api_test_case.evaluation_cost
+                                )
 
                         conversational_test_case.run_duration += (
                             api_test_case.run_duration
@@ -139,7 +144,8 @@ class TestRun(BaseModel):
                 self.evaluation_cost += api_test_case.evaluation_cost
 
     def set_dataset_properties(
-        self, test_case: Union[LLMTestCase, ConversationalTestCase]
+        self,
+        test_case: Union[LLMTestCase, ConversationalTestCase, MLLMTestCase],
     ):
         if self.dataset_alias is None:
             self.dataset_alias = test_case._dataset_alias
@@ -171,8 +177,8 @@ class TestRun(BaseModel):
     def delete_test_case_instance_ids(self):
         for conversational_test_case in self.conversational_test_cases:
             del conversational_test_case.instance_id
-            for message in conversational_test_case.messages:
-                del message.conversational_instance_id
+            for turn in conversational_test_case.turns:
+                del turn.conversational_instance_id
 
         for test_case in self.test_cases:
             del test_case.conversational_instance_id
@@ -207,10 +213,10 @@ class TestRun(BaseModel):
                     else:
                         metrics_dict[name] = [score]
 
-            for message in convo_test_case.messages:
-                if message.metrics_data is None:
+            for turn in convo_test_case.turns:
+                if turn.metrics_data is None:
                     continue
-                for metric_data in message.metrics_data:
+                for metric_data in turn.metrics_data:
                     name = metric_data.name
                     score = metric_data.score
                     if score is None:
@@ -221,7 +227,7 @@ class TestRun(BaseModel):
                     else:
                         metrics_dict[name] = [score]
 
-        # metrics_scores combines both conversational and nonconvo scores
+        # metrics_scores combines both conversational and nonconvo and mllm scores
         # might need to separate in the future
         self.metrics_scores = [
             MetricScores(metric=metric, scores=scores)
@@ -261,7 +267,15 @@ class TestRun(BaseModel):
 
     @classmethod
     def load(cls, f):
-        return cls(**json.load(f))
+        data: dict = json.load(f)
+        return cls(**data)
+
+    def guard_mllm_test_cases(self):
+        for test_case in self.test_cases:
+            if test_case.is_multimodal():
+                raise ValueError(
+                    "Unable to send multimodal test cases to Confident AI."
+                )
 
 
 class TestRunManager:
@@ -282,11 +296,13 @@ class TestRunManager:
 
     def create_test_run(
         self,
+        identifier: Optional[str] = None,
         file_name: Optional[str] = None,
         disable_request: Optional[bool] = False,
     ):
         self.disable_request = disable_request
         test_run = TestRun(
+            identifier=identifier,
             testFile=file_name,
             testCases=[],
             metricsScores=[],
@@ -299,9 +315,9 @@ class TestRunManager:
         if self.save_to_disk:
             self.save_test_run()
 
-    def get_test_run(self):
+    def get_test_run(self, identifier: Optional[str] = None):
         if self.test_run is None:
-            self.create_test_run()
+            self.create_test_run(identifier=identifier)
 
         if self.save_to_disk:
             try:
@@ -334,8 +350,14 @@ class TestRunManager:
     def update_test_run(
         self,
         api_test_case: Union[LLMApiTestCase, ConversationalApiTestCase],
-        test_case: Union[LLMTestCase, ConversationalTestCase],
+        test_case: Union[LLMTestCase, ConversationalTestCase, MLLMTestCase],
     ):
+        if (
+            api_test_case.metrics_data is not None
+            and len(api_test_case.metrics_data) == 0
+        ):
+            return
+
         if self.save_to_disk:
             try:
                 with portalocker.Lock(
@@ -379,6 +401,9 @@ class TestRunManager:
         table.add_column("Overall Success Rate", justify="left")
 
         for index, test_case in enumerate(test_run.test_cases):
+            if test_case.metrics_data is None:
+                continue
+
             pass_count = 0
             fail_count = 0
             test_case_name = test_case.name
@@ -438,47 +463,48 @@ class TestRunManager:
             fail_count = 0
             conversational_test_case_name = conversational_test_case.name
 
-            for metric_data in conversational_test_case.metrics_data:
-                if metric_data.success:
-                    pass_count += 1
-                else:
-                    fail_count += 1
-
-            table.add_row(
-                conversational_test_case_name,
-                "",
-                "",
-                "",
-                f"{round((100*pass_count)/(pass_count+fail_count),2)}%",
-            )
-
-            for metric_data in conversational_test_case.metrics_data:
-                if metric_data.error:
-                    status = "[red]ERRORED[/red]"
-                elif metric_data.success:
-                    status = "[green]PASSED[/green]"
-                else:
-                    status = "[red]FAILED[/red]"
-
-                evaluation_model = metric_data.evaluation_model
-                if evaluation_model is None:
-                    evaluation_model = "n/a"
-
-                if metric_data.score is not None:
-                    metric_score = round(metric_data.score, 2)
-                else:
-                    metric_score = None
-
+            if conversational_test_case.metrics_data is not None:
+                for metric_data in conversational_test_case.metrics_data:
+                    if metric_data.success:
+                        pass_count += 1
+                    else:
+                        fail_count += 1
                 table.add_row(
+                    conversational_test_case_name,
                     "",
-                    str(metric_data.name),
-                    f"{metric_score} (threshold={metric_data.threshold}, evaluation model={evaluation_model}, reason={metric_data.reason}, error={metric_data.error})",
-                    status,
                     "",
+                    "",
+                    f"{round((100*pass_count)/(pass_count+fail_count),2)}%",
                 )
 
-            for test_case in conversational_test_case.messages:
-                if test_case.metrics_data is None:
+            if conversational_test_case.metrics_data is not None:
+                for metric_data in conversational_test_case.metrics_data:
+                    if metric_data.error:
+                        status = "[red]ERRORED[/red]"
+                    elif metric_data.success:
+                        status = "[green]PASSED[/green]"
+                    else:
+                        status = "[red]FAILED[/red]"
+
+                    evaluation_model = metric_data.evaluation_model
+                    if evaluation_model is None:
+                        evaluation_model = "n/a"
+
+                    if metric_data.score is not None:
+                        metric_score = round(metric_data.score, 2)
+                    else:
+                        metric_score = None
+
+                    table.add_row(
+                        "",
+                        str(metric_data.name),
+                        f"{metric_score} (threshold={metric_data.threshold}, evaluation model={evaluation_model}, reason={metric_data.reason}, error={metric_data.error})",
+                        status,
+                        "",
+                    )
+
+            for turn in conversational_test_case.turns:
+                if turn.metrics_data is None:
                     # skip if no evaluation
                     continue
 
@@ -534,14 +560,22 @@ class TestRunManager:
                     "",
                 )
 
+            if index is not len(self.test_run.test_cases) - 1:
+                table.add_row(
+                    "",
+                    "",
+                    "",
+                    "",
+                    "",
+                )
+
         print(table)
         print(
             f"Total estimated evaluation tokens cost: {test_run.evaluation_cost} USD"
         )
 
-    def post_test_run(self, test_run: TestRun):
+    def post_test_run(self, test_run: TestRun) -> Optional[str]:
         console = Console()
-
         if is_confident() and self.disable_request is False:
             BATCH_SIZE = 60
             CONVERSATIONAL_BATCH_SIZE = BATCH_SIZE // 3
@@ -581,6 +615,7 @@ class TestRunManager:
                 endpoint=Endpoints.TEST_RUN_ENDPOINT,
                 body=body,
             )
+
             response = TestRunHttpResponse(
                 testRunId=result["testRunId"],
                 projectId=result["projectId"],
@@ -645,16 +680,17 @@ class TestRunManager:
             console.print(
                 "[rgb(5,245,141)]✓[/rgb(5,245,141)] Tests finished 🎉! View results on "
                 f"[link={link}]{link}[/link]."
-                "\n‼️  NOTE: You can also run evaluations on ALL of deepeval's metrics directly on Confident AI instead."
+                "\n‼️  Friendly reminder 😇: You can also run evaluations with ALL of deepeval's metrics directly on Confident AI instead."
             )
 
             if is_in_ci_env() == False:
                 webbrowser.open(link)
 
+            return link
         else:
             console.print(
-                "[rgb(5,245,141)]✓[/rgb(5,245,141)] Tests finished 🎉! Run 'deepeval login' to view evaluation results on Confident AI. "
-                "\n‼️  NOTE: You can also run evaluations on ALL of deepeval's metrics directly on Confident AI instead."
+                "[rgb(5,245,141)]✓[/rgb(5,245,141)] Tests finished 🎉! Run 'deepeval login' to save and analyze evaluation results on Confident AI. "
+                "\n‼️  Friendly reminder 😇: You can also run evaluations with ALL of deepeval's metrics directly on Confident AI instead."
             )
 
     def save_test_run_locally(self):
@@ -677,7 +713,9 @@ class TestRunManager:
                 print(f"Results saved in {local_folder} as {new_test_filename}")
             os.remove(new_test_filename)
 
-    def wrap_up_test_run(self, runDuration: float, display_table: bool = True):
+    def wrap_up_test_run(
+        self, runDuration: float, display_table: bool = True
+    ) -> Optional[str]:
         test_run = self.get_test_run()
         if test_run is None:
             print("Test Run is empty, please try again.")
@@ -716,7 +754,13 @@ class TestRunManager:
 
         self.save_test_run_locally()
         delete_file_if_exists(self.temp_file_name)
-        self.post_test_run(test_run)
+
+        if (
+            len(test_run.test_cases) > 0
+            or len(test_run.conversational_test_cases) > 0
+        ):
+            test_run.guard_mllm_test_cases()
+            return self.post_test_run(test_run)
 
 
 global_test_run_manager = TestRunManager()
